
local assets=
{
	Asset("ANIM", "anim/gnomenade.zip"),
	Asset("ANIM", "anim/swap_gnomenade.zip"),
	Asset("ATLAS", "images/inventoryimages/gnomenade.xml"),
	Asset("IMAGE", "images/inventoryimages/gnomenade.tex"),
}

local prefabs = 
{
	"collapse_big",
	"groundpound_fx",
    "groundpoundring_fx",
	"explode_large",
	"gnome_print",
}	
 
 local function DoExplode(self)
    local explosiverange = 10
	
    local stacksize = self.inst.components.stackable ~= nil and self.inst.components.stackable:StackSize() or 1
    local totaldamage = self.explosivedamage * stacksize
	
    local x, y, z = self.inst.Transform:GetWorldPosition()
    -- Players are off limits now
    local ents = TheSim:FindEntities(x, y, z, explosiverange, nil, { "INLIMBO", "player" })
 
    for i, v in ipairs(ents) do
        if v ~= self.inst and v:IsValid() and not v:IsInLimbo() then
            if v.components.workable  then
                v.components.workable:WorkedBy(self.inst, self.buildingdamage)
            end
 
            --Recheck valid after work
            if v:IsValid() and not v:IsInLimbo() then
                if self.lightonexplode and
                    v.components.fueled == nil and
                    v.components.burnable ~= nil and
                    not v.components.burnable:IsBurning() and
                    not v:HasTag("burnt") then
                    --v.components.burnable:Ignite()
                end
 
                if v.components.combat ~= nil then
                    v.components.combat:GetAttacked(self.inst, totaldamage, nil)
                end
 
                v:PushEvent("explosion", { explosive = self.inst })
            end
        end
    end
	
end
 
local function OnExplodeFn(inst, scale)
	scale = scale or 1
	local pos = Vector3(inst.Transform:GetWorldPosition())
	inst.AnimState:PlayAnimation("idle", true)
	local explode = SpawnPrefab("explode_large")
	local ring = SpawnPrefab("groundpoundring_fx")
	local pos = inst:GetPosition()

	ring.Transform:SetPosition(pos.x, pos.y, pos.z)
	ring.Transform:SetScale(scale, scale, scale)

	explode.Transform:SetPosition(pos.x, pos.y, pos.z)
	explode.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
	explode.AnimState:SetLightOverride(1)
	explode.Transform:SetScale(scale, scale, scale)
    
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("groundpound_fx").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("groundpoundring_fx").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("gnome_print").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("gnomenade/fall/explode")
	inst.SoundEmitter:PlaySound("gnomenade/fall/laugh")
	
	inst:Remove()
	inst.AnimState:SetLightOverride(1)
	GetPlayer().components.playercontroller:ShakeCamera(inst, "FULL", 0.5, 0.04, .5, 80)
    DoExplode(inst.components.explosive)

end

local function common_fn(bank, build, anim, tag, isinventoryitem)
    local inst = CreateEntity()
 
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

 
    if isinventoryitem then
        MakeInventoryPhysics(inst)
    else
        inst.entity:AddPhysics()
        inst.Physics:SetMass(1)
        inst.Physics:SetCapsule(0.2, 0.2)
        inst.Physics:SetFriction(0)
        inst.Physics:SetDamping(0)
        inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
        inst.Physics:ClearCollisionMask()
        inst.Physics:CollidesWith(COLLISION.WORLD)
    end
 
    if tag ~= nil then
        inst:AddTag(tag)
    end
 
    inst.AnimState:SetBank(bank)
    inst.AnimState:SetBuild(build)
    inst.AnimState:PlayAnimation(anim, true)
 

    inst:AddComponent("locomotor")
 
    --inst:AddComponent("complexprojectile")
 
    return inst
end
 
 
local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_gnomenade", "swap_gnomenade")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end
 
local function onthrown(inst, thrower, pt)
inst.components.fuse:StartFuse()
    inst.persists = false	
	
	inst.Light:Enable(true)
	inst.Light:SetRadius(.8)
    inst.Light:SetFalloff(0.66)
    inst.Light:SetIntensity(0.66)
    inst.Light:SetColour(235 / 255, 255 / 255, 111 / 255)
	
	inst.Physics:SetFriction(1)
	inst.Transform:SetFourFaced()
	inst:FacePoint(pt:Get())
	inst.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
    inst.AnimState:PlayAnimation("spin_loop", true)
	inst.SoundEmitter:PlaySound("gnomenade/fall/prep")

		inst.LightTask = inst:DoPeriodicTask(FRAMES, function()
		local pos = inst:GetPosition()

		if pos.y <= 0.1 then
		inst:DoTaskInTime(2, function()
				if inst and inst.LightTask then
					inst.LightTask:Cancel()
				end
			end)
		end	
    		local rad = math.clamp(Lerp(2, 0, pos.y/6), 0, 2)
    		local intensity = math.clamp(Lerp(0.8, 0.5, pos.y/7), 0.5, 0.8)
	end)
	
end

 
local function fn()
    local inst = common_fn("gnomenade", "gnomenade", "idle", "projectile", true)
	inst.entity:AddSoundEmitter()
	inst.entity:AddLight()
	inst:AddTag("thrown")
	inst:AddTag("projectile")
	
	inst:AddComponent("throwable")
	inst.components.throwable.onthrown = onthrown
 
    inst:AddComponent("explosive")
	inst.components.explosive:SetOnExplodeFn(OnExplodeFn)
	inst.components.explosive.explosivedamage = TUNING.COCONADE_DAMAGE*2
	inst.components.explosive.explosiverange = TUNING.COCONADE_EXPLOSIONRANGE*2
	inst.components.explosive.buildingdamage = TUNING.COCONADE_BUILDINGDAMAGE*2
	
	inst:AddComponent("appeasement")
    inst.components.appeasement.appeasementvalue = TUNING.WRATH_LARGE

    inst:AddComponent("reticule")
    inst.components.reticule.targetfn = function() 
        return inst.components.throwable:GetThrowPoint()
    end
    inst.components.reticule.ease = true
 
    inst:AddComponent("inspectable")
 
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.imagename = "gnomenade"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/gnomenade.xml"
 
	 inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(5)
 
	inst:AddComponent("fuse")
    inst.components.fuse:SetFuseTime(TUNING.COCONADE_FUSE)
    inst.components.fuse.onfusedone = OnExplodeFn
 
    inst:AddComponent("stackable")
 
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
    inst.components.equippable.equipstack = true
 
    return inst
end
 
STRINGS.CHARACTERS.GENERIC.DESCRIBE.GNOMENADE = "I think it's looking at me."

 
return Prefab( "common/inventory/gnomenade", fn, assets) 